var keyconfig_options = {

 oldkey:null,

 set_bindings:null,

 /** Read key from preferences */
 getini: function(key) {
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                     .getService(Components.interfaces.nsIPrefService)
                                     .getBranch("extensions.keyconfig.");
  try {
   return prefService.getCharPref(key+"Key");
  } catch (e) {
   return "";
  }
 },

 /** Store key into preferences */
 setini: function(key,value) {
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                     .getService(Components.interfaces.nsIPrefService)
                                     .getBranch("extensions.keyconfig.");
  return prefService.setCharPref(key+"Key",value);
 },

 /** Store currently edited keyboard shortcut when focus is moved away and on saving */
 store_old: function() {
  if (keyconfig_options.oldkey==null) return;
  var key=document.getElementById('key').value;
  if (key.length==1) key=key.toLowerCase();
  var mod="";
  if (document.getElementById('ctrl').checked) mod+="Ctrl+";
  if (document.getElementById('alt').checked) mod+="Alt+";
  if (document.getElementById('shift').checked) mod+="Shift+";
  if (document.getElementById('meta').checked) mod+="Meta+";
  if (key!="") key=mod+key;
  var ikey=keyconfig_data.keys[keyconfig_options.oldkey];
  this.set_bindings[ikey]=key;
  var tr=keyconfig_options.treeView.treebox;
  tr.invalidateRow(keyconfig_options.oldkey);
 },

 /** Called when key entry changes */
 update_selected: function() {
  keyconfig_options.store_old();
 },

 /** Called when one of the checkboxes changes */
 update_cbox: function(cbox) {
  cbox.checked=!cbox.checked;
  keyconfig_options.store_old();
  cbox.checked=!cbox.checked;
 },

 /** Return current value of keyboard shortcut (the once currently edited in dialog) */
 curvalue: function(key) {
  var w=this.set_bindings[key];
  if (w!=undefined && w!=null) return w;
  return keyconfig_options.getini(key);
 },

 /** Called on changing selection in treeview */
 select: function(tr) {
  var i=tr.currentIndex;
  keyconfig_options.store_old();
  var k=keyconfig_data.keys[i];
  var v=keyconfig_options.curvalue(k);
  var key=document.getElementById('key');
  var ctrl=document.getElementById('ctrl');
  var alt=document.getElementById('alt');
  var shift=document.getElementById('shift');
  var meta=document.getElementById('meta');
  if (v.match("^Ctrl\\+"))  {v=v.substring(5);ctrl.checked=true;} else {ctrl.checked=false;}
  if (v.match("^Alt\\+"))   {v=v.substring(4);alt.checked=true;} else {alt.checked=false;}
  if (v.match("^Shift\\+")) {v=v.substring(6);shift.checked=true;} else {shift.checked=false;}
  if (v.match("^Meta\\+"))  {v=v.substring(5);meta.checked=true;} else {meta.checked=false;}
  key.value=v;
  keyconfig_options.oldkey=i;
 },

 /** Get localized name of action */
 actionName : function(ac) {
  var prop=document.getElementById("keyconfig-string-bundle");
  try {
   var s=prop.getString("keyconfig_"+ac);
   if (s==null || s==undefined || s=="") return ac;
  } catch (e) {
   return ac;
  }
  return s;
 },

 /** Get localized string */
 _ : function(ac) {
  var prop=document.getElementById("keyconfig-string-bundle");
  try {
   var s=prop.getString("_"+ac);
   if (s==null || s==undefined || s=="") return ac;
  } catch (e) {
   return ac;
  }
  return s;
 },

 /** Treeview model */
 treeView : {
  rowCount : keyconfig_data.keys.length,
  getCellText : function(row,column){
   var k=keyconfig_data.keys[row]
    if (column.id=="action") return keyconfig_options.actionName(k);
    var v=keyconfig_options.curvalue(k);
    return v;
  },
  setTree: function(treebox){ this.treebox = treebox; },
  isContainer: function(row){ return false; },
  isSeparator: function(row){ return false; },
  isSorted: function(){ return false; },
  getLevel: function(row){ return 0; },
  getImageSrc: function(row,col){ return null; },
  getRowProperties: function(row,props){},
  getCellProperties: function(row,col,props){},
  getColumnProperties: function(colid,col,props){}
 },

 /** Initializes the options */
 initialize: function() {
  keyconfig_options.oldkey=null;
  this.set_bindings=new Array();
  document.getElementById('keylist').view = keyconfig_options.treeView;
 },

 /** Uninitializes the options */
 uninitialize: function() {
 },

 getFilePicker: function(mode) {
  var nsIFilePicker=Components.interfaces.nsIFilePicker;
  var fp=Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
  fp.defaultExtension="kbconfig";
  fp.defaultString="custom.kbconfig";
  if (mode=="load") {
   fp.init(window, keyconfig_options._("Select a filename to load"), nsIFilePicker.modeOpen);
  } else {
   fp.init(window, keyconfig_options._("Select a filename to save"), nsIFilePicker.modeSave);
  }
  fp.appendFilter(keyconfig_options._("Keyboard configs"),"*.ini; *.kbconfig");
  var res = fp.show();
  if (res == nsIFilePicker.returnOK || res == nsIFilePicker.returnReplace){
   return fp.file;
  }
  return null;
 },

 /** Load configuration from a file */
 loadconfig: function() {
  var fp=keyconfig_options.getFilePicker("load");
  if (fp==null) return;
  if (!fp.exists()) return;

  try {
    var s=Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
    s.init(fp,1,0444,0);
    s.QueryInterface(Components.interfaces.nsILineInputStream);
    var ok, line = {};
    do {
      ok = s.readLine(line);
      var str=line.value;
      //TODO str -> process line
    } while(ok);
    s.close();
  } catch (e) {
    alert("Unable to load file "+fp);
  }
 },

 /** Save configuration to a file */
 saveconfig: function() {
  var fp=keyconfig_options.getFilePicker("save");
  if (fp==null) return;
  /*fp.create(fp.NORMAL_FILE_TYPE,0644);
  if (!fp.exists()) {
   alert(keyconfig_options._("Unable to open file for writing"));
   return;
  }*/

  try {
   var s=Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
   s.init(fp, 0x02 | 0x08 | 0x20, 0644, 0);
   //TODO data = ...
   var data="";
   keyconfig_options.store_old();
   alert(keyconfig_data.keys.length);
   for (var i=0;i<keyconfig_data.keys.length;i++) {
    var k=keyconfig_data.keys[i];
    var v=this.set_bindings[k];
    if (v==null) v=keyconfig_options.getini(k);
    if (v==null || v=="") continue;
    data+=k+"="+v+"\n";
   }
  alert(data);
  alert(data.length)
   s.write(data,data.length);
   s.close();
  } catch (e) {
   alert("Unable to save to file "+e);
  }
 },

 /** Saves the options from dialog */
 saveOptions: function() {
  keyconfig_options.store_old();
  for (var i=0;i<keyconfig_data.keys.length;i++) {
   var k=keyconfig_data.keys[i];
   var v=this.set_bindings[k];
   if (v==null) continue;
   try {
    keyconfig_options.setini(k,v);
   } catch (e) {
    alert("Exception when saving options:"+e);
   }
  }
  return true;
 },

};
